@@ -1,4 +1,4 @@
-/* $Id: economy.cpp 12209 2008-02-21 19:09:10Z peter1138 $ */
+/* $Id: economy.cpp 13685 2008-07-09 18:59:39Z rubidium $ */
 
 /** @file economy.cpp */
 
@@ -45,6 +45,7 @@
 #include "rail_map.h"
 #include "signal_func.h"
 #include "gfx_func.h"
+#include "autoreplace_func.h"
 
 #include "table/strings.h"
 #include "table/sprites.h"
@@ -455,6 +456,19 @@ void ChangeOwnershipOfPlayerItems(PlayerID old_player, PlayerID new_player)
 		UpdateSignalsInBuffer();
 	}
 
+	/* In all cases clear replace engine rules.
+	 * Even if it was copied, it could interfere with new owner's rules */
+	RemoveAllEngineReplacementForPlayer(GetPlayer(old_player));
+
+	if (new_player == PLAYER_SPECTATOR) {
+		RemoveAllGroupsForPlayer(old_player);
+	} else {
+		Group *g;
+		FOR_ALL_GROUPS(g) {
+			if (g->owner == old_player) g->owner = new_player;
+		}
+	}
+
 	/* Change color of existing windows */
 	if (new_player != PLAYER_SPECTATOR) ChangeWindowOwner(old_player, new_player);
 
@@ -1597,7 +1611,7 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 		byte load_amount = EngInfo(v->engine_type)->load_amount;
 		if (_patches.gradual_loading && HasBit(EngInfo(v->engine_type)->callbackmask, CBM_VEHICLE_LOAD_AMOUNT)) {
 			uint16 cb_load_amount = GetVehicleCallback(CBID_VEHICLE_LOAD_AMOUNT, 0, 0, v->engine_type, v);
-			if (cb_load_amount != CALLBACK_FAILED && cb_load_amount != 0) load_amount = cb_load_amount & 0xFF;
+			if (cb_load_amount != CALLBACK_FAILED && GB(cb_load_amount, 0, 8) != 0) load_amount = GB(cb_load_amount, 0, 8);
 		}
 
 		GoodsEntry *ge = &st->goods[v->cargo_type];
@@ -1997,7 +2011,7 @@ static const SaveLoad _economy_desc[] = {
 	SLE_CONDVAR(Economy, max_loan_unround, SLE_FILE_I32 | SLE_VAR_I64,  0, 64),
 	SLE_CONDVAR(Economy, max_loan_unround, SLE_INT64,                  65, SL_MAX_VERSION),
 	SLE_CONDVAR(Economy, max_loan_unround_fract, SLE_UINT16,           70, SL_MAX_VERSION),
-	    SLE_VAR(Economy, fluct,            SLE_FILE_I16 | SLE_VAR_I32),
+	    SLE_VAR(Economy, fluct,            SLE_INT16),
 	    SLE_VAR(Economy, interest_rate,    SLE_UINT8),
 	    SLE_VAR(Economy, infl_amount,      SLE_UINT8),
 	    SLE_VAR(Economy, infl_amount_pr,   SLE_UINT8),
