@@ -1,4 +1,4 @@
-/* $Id: newgrf_station.cpp 12368 2008-03-15 13:21:31Z smatz $ */
+/* $Id: newgrf_station.cpp 13871 2008-07-29 22:37:54Z rubidium $ */
 
 /** @file newgrf_station.cpp Functions for dealing with station classes and custom stations. */
 
@@ -17,11 +17,12 @@
 #include "newgrf_station.h"
 #include "newgrf_spritegroup.h"
 #include "cargotype.h"
-#include "town_map.h"
+#include "town.h"
 #include "newgrf_town.h"
 #include "gfx_func.h"
 #include "date_func.h"
 #include "player_func.h"
+#include "tunnelbridge_map.h"
 
 #include "table/sprites.h"
 #include "table/strings.h"
@@ -291,11 +292,18 @@ static uint32 GetRailContinuationInfo(TileIndex tile)
 	uint i;
 
 	for (i = 0; i < lengthof(x_dir); i++, dir++, diagdir++) {
-		TrackBits trackbits = TrackStatusToTrackBits(GetTileTrackStatus(tile + TileOffsByDir(*dir), TRANSPORT_RAIL, 0));
+		TileIndex neighbour_tile = tile + TileOffsByDir(*dir);
+		TrackBits trackbits = TrackStatusToTrackBits(GetTileTrackStatus(neighbour_tile, TRANSPORT_RAIL, 0));
 		if (trackbits != TRACK_BIT_NONE) {
 			/* If there is any track on the tile, set the bit in the second byte */
 			SetBit(res, i + 8);
 
+			/* With tunnels and bridges the tile has tracks, but they are not necessarily connected
+			 * with the next tile because the ramp is not going in the right direction. */
+			if (IsTileType(neighbour_tile, MP_TUNNELBRIDGE) && GetTunnelBridgeDirection(neighbour_tile) != *diagdir) {
+				continue;
+			}
+
 			/* If any track reaches our exit direction, set the bit in the lower byte */
 			if (trackbits & DiagdirReachesTracks(*diagdir)) SetBit(res, i);
 		}
@@ -350,12 +358,12 @@ static uint32 StationGetVariable(const ResolverObject *object, byte variable, by
 
 	if (object->scope == VSG_SCOPE_PARENT) {
 		/* Pass the request on to the town of the station */
-		Town *t;
+		const Town *t;
 
 		if (st != NULL) {
 			t = st->town;
 		} else if (tile != INVALID_TILE) {
-			t = GetTownByTile(tile);
+			t = ClosestTownFromTile(tile, UINT_MAX);
 		} else {
 			*available = false;
 			return UINT_MAX;
@@ -665,7 +673,7 @@ int AllocateSpecToStation(const StationSpec *statspec, Station *st, bool exec)
 {
 	uint i;
 
-	if (statspec == NULL) return 0;
+	if (statspec == NULL || st == NULL) return 0;
 
 	/* Check if this spec has already been allocated */
 	for (i = 1; i < st->num_specs && i < MAX_SPECLIST; i++) {
@@ -753,7 +761,7 @@ bool DrawStationTile(int x, int y, RailType railtype, Axis axis, StationClassID
 	const RailtypeInfo *rti = GetRailTypeInfo(railtype);
 	SpriteID relocation;
 	SpriteID image;
-	SpriteID pal = PLAYER_SPRITE_COLOR(_local_player);
+	SpriteID palette = PLAYER_SPRITE_COLOR(_local_player);
 	uint tile = 2;
 
 	statspec = GetCustomStationSpec(sclass, station);
@@ -791,6 +799,17 @@ bool DrawStationTile(int x, int y, RailType railtype, Axis axis, StationClassID
 			image += relocation;
 		}
 
+		SpriteID pal;
+		if (HasBit(image, PALETTE_MODIFIER_TRANSPARENT) || HasBit(image, PALETTE_MODIFIER_COLOR)) {
+			if (seq->image.pal > 0) {
+				pal = seq->image.pal;
+			} else {
+				pal = palette;
+			}
+		} else {
+			pal = PAL_NONE;
+		}
+
 		if ((byte)seq->delta_z != 0x80) {
 			pt = RemapCoords(seq->delta_x, seq->delta_y, seq->delta_z);
 			DrawSprite(image, pal, x + pt.x, y + pt.y);
