@@ -1,4 +1,4 @@
-/* $Id: oldloader.cpp 12416 2008-03-25 14:53:59Z glx $ */
+/* $Id: oldloader.cpp 13916 2008-08-01 19:37:10Z rubidium $ */
 
 /** @file oldloader.cpp */
 
@@ -324,6 +324,8 @@ static void FixOldStations()
 	}
 }
 
+static StringID *_old_vehicle_names = NULL;
+
 static void FixOldVehicles()
 {
 	/* Check for shared orders, and link them correctly */
@@ -332,6 +334,8 @@ static void FixOldVehicles()
 	FOR_ALL_VEHICLES(v) {
 		Vehicle *u;
 
+		v->name = CopyFromOldName(_old_vehicle_names[v->index]);
+
 		/* We haven't used this bit for stations for ages */
 		if (v->type == VEH_ROAD &&
 				v->u.road.state != RVSB_IN_DEPOT &&
@@ -417,6 +421,8 @@ static void ReadTTDPatchFlags()
 	and 1 becomes 2. The rest of the values are okay */
 	if (_old_vehicle_multiplier < 2) _old_vehicle_multiplier++;
 
+	_old_vehicle_names = MallocT<StringID>(_old_vehicle_multiplier * 850);
+
 	/* TTDPatch increases the Vehicle-part in the middle of the game,
 	so if the multipler is anything else but 1, the assert fails..
 	bump the assert value so it doesn't!
@@ -993,9 +999,13 @@ static bool LoadOldPlayer(LoadgameState *ls, int num)
 
 	if (!LoadChunk(ls, p, player_chunk)) return false;
 
+	if (_old_string_id == 0) {
+		p->is_active = false;
+		return true;
+	}
+
 	p->name_1 = RemapOldStringID(_old_string_id);
 	p->president_name_1 = RemapOldStringID(_old_string_id_2);
-	p->player_money = p->player_money;
 	_players_ai[_current_player_id].tick = ai_tick;
 
 	if (num == 0) {
@@ -1261,8 +1271,8 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 
 		if (_old_next_ptr != 0xFFFF) v->next = GetVehiclePoolSize() <= _old_next_ptr ? new (_old_next_ptr) InvalidVehicle() : GetVehicle(_old_next_ptr);
 
-		_old_string_id = RemapOldStringID(_old_string_id);
-		v->name = CopyFromOldName(_old_string_id);
+		_old_vehicle_names[_current_vehicle_id] = RemapOldStringID(_old_string_id);
+		v->name = NULL;
 
 		/* Vehicle-subtype is different in TTD(Patch) */
 		if (v->type == VEH_SPECIAL) v->subtype = v->subtype >> 1;
@@ -1540,7 +1550,7 @@ static const OldChunks main_chunk[] = {
 
 	OCL_VAR ( OC_FILE_U32 | OC_VAR_I64,   1, &_economy.max_loan ),
 	OCL_VAR ( OC_FILE_U32 | OC_VAR_I64,   1, &_economy.max_loan_unround ),
-	OCL_VAR ( OC_FILE_U16 | OC_VAR_U32,   1, &_economy.fluct ),
+	OCL_VAR (  OC_INT16,    1, &_economy.fluct ),
 
 	OCL_VAR ( OC_UINT16,    1, &_disaster_delay ),
 
@@ -1602,8 +1612,10 @@ static bool LoadOldMain(LoadgameState *ls)
 
 	DEBUG(oldloader, 3, "Reading main chunk...");
 	/* Load the biggest chunk */
+	_old_vehicle_names = NULL;
 	if (!LoadChunk(ls, NULL, main_chunk)) {
 		DEBUG(oldloader, 0, "Loading failed");
+		free(_old_vehicle_names);
 		return false;
 	}
 	DEBUG(oldloader, 3, "Done, converting game data...");
@@ -1645,7 +1657,13 @@ static bool LoadOldMain(LoadgameState *ls)
 				 * clear it for ourselves and let OTTD's rebuild PBS itself */
 				_m[i].m4 &= 0xF; /* Only keep the lower four bits; upper four is PBS */
 				break;
-			default: break;
+
+			case MP_WATER:
+				if (GetWaterClass(i) == 3) MakeRiver(i, Random());
+				break;
+
+			default:
+				break;
 		}
 	}
 
@@ -1670,6 +1688,8 @@ static bool LoadOldMain(LoadgameState *ls)
 	DEBUG(oldloader, 3, "Finished converting game data");
 	DEBUG(oldloader, 1, "TTD(Patch) savegame successfully converted");
 
+	free(_old_vehicle_names);
+
 	return true;
 }
 
@@ -1694,6 +1714,12 @@ bool LoadOldSaveGame(const char *file)
 
 	fclose(ls.file);
 
+	/* Some old TTDP savegames could have buoys at tile 0
+	 * (without assigned station struct)
+	 * MakeWater() can be used as long as sea has the same
+	 * format as old savegames (eg. everything is zeroed) */
+	MakeWater(0);
+
 	_pause_game = 2;
 
 	return true;
