@@ -1,4 +1,4 @@
-/* $Id: openttd.cpp 12381 2008-03-18 12:28:21Z skidd13 $ */
+/* $Id: openttd.cpp 13871 2008-07-29 22:37:54Z rubidium $ */
 
 /** @file openttd.cpp */
 
@@ -410,13 +410,23 @@ int ttd_main(int argc, char *argv[])
 		case 'i': _use_dos_palette = true; break;
 		case 'g':
 			if (mgo.opt != NULL) {
-				strcpy(_file_to_saveload.name, mgo.opt);
+				ttd_strlcpy(_file_to_saveload.name, mgo.opt, sizeof(_file_to_saveload.name));
 				_switch_mode = SM_LOAD;
-			} else {
-				_switch_mode = SM_NEWGAME;
-				/* Give a random map */
-				generation_seed = InteractiveRandom();
+				_file_to_saveload.mode = SL_LOAD;
+
+				/* if the file doesn't exist or it is not a valid savegame, let the saveload code show an error */
+				const char *t = strrchr(_file_to_saveload.name, '.');
+				if (t != NULL) {
+					FiosType ft = (FiosType)FiosGetSavegameListCallback(SLD_LOAD_GAME, _file_to_saveload.name, t, NULL);
+					if (ft != FIOS_TYPE_INVALID) SetFiosType(ft);
+				}
+
+				break;
 			}
+
+			_switch_mode = SM_NEWGAME;
+			/* Give a random map */
+			generation_seed = InteractiveRandom();
 			break;
 		case 'G': generation_seed = atoi(mgo.opt); break;
 		case 'c': _config_file = strdup(mgo.opt); break;
@@ -687,6 +697,8 @@ static void ShowScreenshotResult(bool b)
 
 static void MakeNewGameDone()
 {
+	SettingsDisableElrail(_patches.disable_elrails);
+
 	/* In a dedicated server, the server does not play */
 	if (_network_dedicated) {
 		SetLocalPlayer(PLAYER_SPECTATOR);
@@ -700,7 +712,6 @@ static void MakeNewGameDone()
 	_current_player = _local_player;
 	DoCommandP(0, (_patches.autorenew << 15 ) | (_patches.autorenew_months << 16) | 4, _patches.autorenew_money, NULL, CMD_SET_AUTOREPLACE);
 
-	SettingsDisableElrail(_patches.disable_elrails);
 	SetDefaultRailGui();
 
 #ifdef ENABLE_NETWORK
@@ -897,6 +908,9 @@ void SwitchMode(int new_mode)
 			SetDParamStr(0, GetSaveLoadErrorString());
 			ShowErrorMessage(INVALID_STRING_ID, STR_012D, 0, 0);
 		} else {
+			if (_saveload_mode == SLD_LOAD_SCENARIO) {
+				StartupEngines();
+			}
 			/* Update the local player for a loaded game. It is either always
 			 * player #1 (eg 0) or in the case of a dedicated server a spectator */
 			SetLocalPlayer(_network_dedicated ? PLAYER_SPECTATOR : PLAYER_FIRST);
@@ -1327,6 +1341,8 @@ static bool InitializeWindowsAndCaches()
 		players[v->owner]->num_engines[v->engine_type]++;
 	}
 
+	CheckTrainsLengths();
+
 	return true;
 }
 
@@ -1808,7 +1824,7 @@ bool AfterLoadGame()
 		}
 
 		FOR_ALL_VEHICLES(v) {
-			if (v->type == VEH_TRAIN && (IsFrontEngine(v) || IsFreeWagon(v))) TrainConsistChanged(v);
+			if (v->type == VEH_TRAIN && (IsFrontEngine(v) || IsFreeWagon(v))) TrainConsistChanged(v, true);
 		}
 
 	}
@@ -2333,9 +2349,7 @@ bool AfterLoadGame()
 
 			/* Move river flag and update canals to use water class */
 			if (IsTileType(t, MP_WATER)) {
-				if (_m[t].m5 == 2) {
-					MakeRiver(t, Random());
-				} else {
+				if (GetWaterClass(t) != WATER_CLASS_RIVER) {
 					if (IsWater(t)) {
 						Owner o = GetTileOwner(t);
 						if (o == OWNER_WATER) {
@@ -2402,13 +2416,13 @@ bool AfterLoadGame()
 		}
 
 		/* Convert old PF settings to new */
-		if (_patches.yapf.rail_use_yapf) {
+		if (_patches.yapf.rail_use_yapf || CheckSavegameVersion(28)) {
 			_patches.pathfinder_for_trains = VPF_YAPF;
 		} else {
 			_patches.pathfinder_for_trains = (_patches.new_pathfinding_all ? VPF_NPF : VPF_NTP);
 		}
 
-		if (_patches.yapf.road_use_yapf) {
+		if (_patches.yapf.road_use_yapf || CheckSavegameVersion(28)) {
 			_patches.pathfinder_for_roadvehs = VPF_YAPF;
 		} else {
 			_patches.pathfinder_for_roadvehs = (_patches.new_pathfinding_all ? VPF_NPF : VPF_OPF);
@@ -2440,6 +2454,21 @@ bool AfterLoadGame()
 		}
 	}
 
+	if (CheckSavegameVersion(62)) {
+		/* Remove all trams from savegames without tram support.
+		 * There would be trams without tram track under causing crashes sooner or later. */
+		Vehicle *v;
+		FOR_ALL_VEHICLES(v) {
+			if (v->type == VEH_ROAD && v->First() == v &&
+					HasBit(EngInfo(v->engine_type)->misc_flags, EF_ROAD_TRAM)) {
+				if (_switch_mode_errorstr == INVALID_STRING_ID || _switch_mode_errorstr == STR_NEWGRF_COMPATIBLE_LOAD_WARNING) {
+					_switch_mode_errorstr = STR_LOADGAME_REMOVED_TRAMS;
+				}
+				delete v;
+			}
+		}
+	}
+
 	return InitializeWindowsAndCaches();
 }
 
@@ -2465,4 +2494,5 @@ void ReloadNewGRFData()
 	UpdateHousesAndTowns();
 	/* redraw the whole screen */
 	MarkWholeScreenDirty();
+	CheckTrainsLengths();
 }
