@@ -1,4 +1,4 @@
-/* $Id: wnd_quartz.mm 11718 2007-12-29 05:15:13Z egladil $ */
+/* $Id: wnd_quartz.mm 12868 2008-04-24 12:32:37Z rubidium $ */
 
 /******************************************************************************
  *                             Cocoa video driver                             *
@@ -175,7 +175,8 @@ static CGColorSpaceRef QZ_GetCorrectColorSpace()
 			CMCloseProfile(sysProfile);
 		}
 
-		assert(colorSpace != NULL);
+		if (colorSpace == NULL)
+			error("Could not get system colour space. You might need to recalibrate your monitor.");
 	}
 
 	return colorSpace;
